"""Functions for displaying the Real Python feed."""

# Standard library imports
from typing import List


def show(article: str) -> None:
    """Show one article."""
    print(article)


def show_list(site: str, titles: List[str]) -> None:
    """Show list of articles."""
    print(f"The latest tutorials from {site}")
    for article_id, title in enumerate(titles):
        print(f"{article_id:>3} {title}")



"""Interact with the Real Python feed."""
# Standard library imports
from typing import Dict, List  # noqa

# Third party imports
import feedparser
import html2text

# Reader imports
from reader import URL

_CACHED_FEEDS: Dict[str, feedparser.FeedParserDict] = {}


def _feed(url: str = URL) -> feedparser.FeedParserDict:
    """Cache contents of the feed, so it's only read once."""
    if url not in _CACHED_FEEDS:
        _CACHED_FEEDS[url] = feedparser.parse(url)
    return _CACHED_FEEDS[url]


def get_site(url: str = URL) -> str:
    """Get name and link to web site of the feed."""
    info = _feed(url).feed
    return f"{info.title} ({info.link})"


def get_article(article_id: str, links: bool = False, url: str = URL) -> str:
    """Get article from feed with the given ID."""
    articles = _feed(url).entries
    try:
        article = articles[int(article_id)]
    except (IndexError, ValueError):
        max_id = len(articles) - 1
        msg = f"Unknown article ID, use ID from 0 to {max_id}"
        raise SystemExit(f"Error: {msg}")

    # Get article as HTML
    try:
        html = article.content[0].value
    except AttributeError:
        html = article.summary

    # Convert HTML to plain text
    to_text = html2text.HTML2Text()
    to_text.ignore_links = not links
    text = to_text.handle(html)

    return f"# {article.title}\n\n{text}"


def get_titles(url: str = URL) -> List[str]:
    """List titles in feed."""
    articles = _feed(url).entries
    return [a.title for a in articles]



"""Real Python feed reader.

Import the `feed` module to work with the Real Python feed:

    >>> from reader import feed
    >>> feed.get_titles()
    ['Logging in Python', 'The Best Python Books', ...]

See https://github.com/realpython/reader/ for more information.
"""
from configparser import ConfigParser
from importlib import resources

# Version of realpython-reader package
__version__ = "1.0.75"

# Read URL of feed from config file
cfg = ConfigParser()
with resources.path("reader", "config.cfg") as path:
    cfg.read(str(path))

URL = cfg.get("feed", "url")



"""Setup script for mytest-pythonreader2"""

# Standard library imports
import pathlib

# Third party imports
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).resolve().parent

# The text of the README file is used as a description
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="mytest-pythonreader2",
    version="1.0.75",
    description="Read the latest Real Python tutorials",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/realpython/reader",
    author="Real Python",
    author_email="info@realpython.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
    ],
    packages=["reader"],
    include_package_data=True,
    install_requires=["feedparser", "html2text"],
    entry_points={"console_scripts": ["realpython=reader.__main__:main"]},
)
